@@ -52,10 +52,11 @@ static void *timer_handler(void *arg __attribute__((unused)));
 */
 
 static int compare_timespec(void *not_used __attribute__((unused)),
-                            uchar *a_ptr, uchar *b_ptr)
+                            const void *a_ptr, const void *b_ptr)
 {
-  return cmp_timespec((*(struct timespec*) a_ptr),
-                      (*(struct timespec*) b_ptr));
+  const struct timespec *ap= a_ptr;
+  const struct timespec *bp= b_ptr;
+  return cmp_timespec((*ap), (*bp));
 }
 
 
@@ -336,8 +337,7 @@ static void *timer_handler(void *arg __attribute__((unused)))
   }
   mysql_mutex_unlock(&LOCK_timer);
   my_thread_end();
-  pthread_exit(0);
-  return 0;					/* Impossible */
+  return 0;
 }
 
 
