@@ -209,6 +209,7 @@ INNODB_DATA_HOME_DIR=$(trim_dir "${INNODB_DATA_HOME_DIR:-}")
 INNODB_LOG_GROUP_HOME=$(trim_dir "${INNODB_LOG_GROUP_HOME:-}")
 INNODB_UNDO_DIR=$(trim_dir "${INNODB_UNDO_DIR:-}")
 INNODB_BUFFER_POOL=""
+INNODB_BUFFER_POOL_SIZE=""
 INNODB_FORCE_RECOVERY=""
 INNOEXTRA=""
 
@@ -329,6 +330,10 @@ case "$1" in
         readonly INNODB_BUFFER_POOL=$(trim_string "$2")
         shift
         ;;
+    '--innodb-buffer-pool-size')
+        readonly INNODB_BUFFER_POOL_SIZE=$(trim_string "$2")
+        shift
+        ;;
     '--defaults-file')
         file=$(trim_string "$2")
         readonly WSREP_SST_OPT_DEFAULT="$1=$file"
@@ -607,6 +612,12 @@ case "$1" in
                        fi
                        skip_mysqld_arg=1
                        ;;
+                   '--innodb-buffer-pool-size')
+                       if [ -z "$INNODB_BUFFER_POOL_SIZE" ]; then
+                           MYSQLD_OPT_INNODB_BUFFER_POOL_SIZE=$(trim_string "$value")
+                       fi
+                       skip_mysqld_arg=1
+                       ;;
                    '--innodb-force-recovery')
                        if [ -n "$value" -a "$value" != "0" ]; then
                            INNODB_FORCE_RECOVERY=$(trim_string "$value")
@@ -696,6 +707,10 @@ if [ -n "${MYSQLD_OPT_INNODB_BUFFER_POOL:-}" -a \
      -z "$INNODB_BUFFER_POOL" ]; then
     readonly INNODB_BUFFER_POOL="$MYSQLD_OPT_INNODB_BUFFER_POOL"
 fi
+if [ -n "${MYSQLD_OPT_INNODB_BUFFER_POOL_SIZE:-}" -a \
+     -z "$INNODB_BUFFER_POOL_SIZE" ]; then
+    readonly INNODB_BUFFER_POOL_SIZE="$MYSQLD_OPT_INNODB_BUFFER_POOL_SIZE"
+fi
 if [ -n "${MYSQLD_OPT_LOG_BIN:-}" -a \
      -z "$WSREP_SST_OPT_BINLOG" ]; then
     readonly WSREP_SST_OPT_BINLOG="$MYSQLD_OPT_LOG_BIN"
@@ -752,6 +767,9 @@ fi
 if [ -n "$INNODB_BUFFER_POOL" ]; then
     INNOEXTRA="$INNOEXTRA --innodb-buffer-pool-filename='$INNODB_BUFFER_POOL'"
 fi
+if [ -n "$INNODB_BUFFER_POOL_SIZE" ]; then
+    INNOEXTRA="$INNOEXTRA --innodb-buffer-pool-size='$INNODB_BUFFER_POOL_SIZE'"
+fi
 if [ -n "$WSREP_SST_OPT_BINLOG" ]; then
     INNOEXTRA="$INNOEXTRA --log-bin='$WSREP_SST_OPT_BINLOG'"
     if [ -n "$WSREP_SST_OPT_BINLOG_INDEX" ]; then
@@ -1154,7 +1172,7 @@ get_openssl()
 {
     # If the OPENSSL_BINARY variable is already defined, just return:
     if [ -n "${OPENSSL_BINARY+x}" ]; then
-        return
+        return 0
     fi
     # Let's look for openssl:
     OPENSSL_BINARY=$(commandex 'openssl')
@@ -1593,7 +1611,7 @@ cleanup_pid()
                     fi
                 done
             elif ps -p $pid >/dev/null 2>&1; then
-                wsrep_log_warning "Unable to kill PID=$pid ($pid_file)"
+                wsrep_log_warning "Unable to kill PID=$pid${pid_file:+ ($pid_file)}"
                 return 1
             fi
         fi
@@ -1812,14 +1830,14 @@ simple_cleanup()
 create_data()
 {
     OLD_PWD="$(pwd)"
+    DATA_DIR="$OLD_PWD"
 
     if [ -n "$DATA" -a "$DATA" != '.' ]; then
         [ ! -d "$DATA" ] && mkdir -p "$DATA"
         cd "$DATA"
+        DATA_DIR="$(pwd)"
+        cd "$OLD_PWD"
     fi
-    DATA_DIR="$(pwd)"
-
-    cd "$OLD_PWD"
 }
 
 create_dirs()
@@ -1900,6 +1918,8 @@ create_dirs()
         cd "$OLD_PWD"
         [ $simplify -ne 0 -a "$ar_log_dir" = "$DATA_DIR" ] && ar_log_dir=""
     fi
+
+    return 0
 }
 
 wait_previous_sst()
