@@ -664,6 +664,31 @@ public:
 }; // class sp_instr_set : public sp_lex_instr
 
 
+/*
+  This instr initializes parameters with default values
+  if it's parameter's spvar was not set by caller.
+*/
+class sp_instr_set_default_param : public sp_instr_set
+{
+  /**< Prevent use of these */
+  sp_instr_set_default_param(const sp_instr_set_default_param &);
+  void operator=(sp_instr_set_default_param &);
+
+public:
+  sp_instr_set_default_param(uint ip, sp_pcontext *ctx,
+               const Sp_rcontext_handler *rh,
+	             uint offset, Item *val,
+               LEX *lex, bool lex_resp,
+	             const LEX_CSTRING &expr_str)
+    : sp_instr_set(ip, ctx, rh, offset, val, lex, lex_resp, expr_str)
+  {}
+
+  virtual ~sp_instr_set_default_param() = default;
+  int execute(THD *thd, uint *nextp) override;
+  void print(String *str) override;
+};
+
+
 /*
   This class handles assignments of a ROW fields:
     DECLARE rec ROW (a INT,b INT);
