@@ -314,13 +314,14 @@ public:
 
   /** Latch a buffer pool block.
   @param block    block to be latched
-  @param rw_latch RW_S_LATCH, RW_SX_LATCH, RW_X_LATCH, RW_NO_LATCH */
-  void page_lock(buf_block_t *block, ulint rw_latch);
+  @param rw_latch RW_S_LATCH, RW_SX_LATCH, RW_X_LATCH, RW_NO_LATCH
+  @return block */
+  buf_block_t *page_lock(buf_block_t *block, ulint rw_latch) noexcept;
 
   /** Acquire a latch on a buffer-fixed buffer pool block.
   @param savepoint   savepoint location of the buffer-fixed block
   @param rw_latch    latch to acquire */
-  void upgrade_buffer_fix(ulint savepoint, rw_lock_type_t rw_latch);
+  void upgrade_buffer_fix(ulint savepoint, rw_lock_type_t rw_latch) noexcept;
 
   /** Register a change to the page latch state. */
   void lock_register(ulint savepoint, mtr_memo_type_t type)
@@ -331,8 +332,10 @@ public:
     slot.type= type;
   }
 
-  /** Upgrade U locks on a block to X */
-  void page_lock_upgrade(const buf_block_t &block);
+  /** Upgrade U locks on a block to X
+  @param block   block on which to upgrade
+  @return &block */
+  buf_block_t *page_lock_upgrade(const buf_block_t &block) noexcept;
 
   /** Upgrade index U lock to X */
   ATTRIBUTE_COLD void index_lock_upgrade();
@@ -681,7 +684,7 @@ private:
 
   /** Write a FILE_MODIFY record when a non-predefined persistent
   tablespace was modified for the first time since fil_names_clear(). */
-  ATTRIBUTE_NOINLINE ATTRIBUTE_COLD void name_write();
+  ATTRIBUTE_NOINLINE ATTRIBUTE_COLD void name_write() noexcept;
 
   /** Encrypt the log */
   ATTRIBUTE_NOINLINE void encrypt();
