@@ -100,6 +100,7 @@ void log_t::create()
   ut_ad(!checkpoint_buf);
   ut_ad(!buf);
   ut_ad(!flush_buf);
+  ut_ad(!writer);
   max_buf_free= 1;
 
   latch.SRW_LOCK_INIT(log_latch_key);
@@ -185,7 +186,6 @@ void log_file_t::write(os_offset_t offset, span<const byte> buf) noexcept
   abort();
 }
 
-#ifdef HAVE_INNODB_MMAP
 # ifdef HAVE_PMEM
 #  include "cache.h"
 # endif
@@ -201,6 +201,10 @@ static void *log_mmap(os_file_t file,
 # endif
                       os_offset_t size)
 {
+#if SIZEOF_SIZE_T < 8
+  if (size != os_offset_t(size_t(size)))
+    return MAP_FAILED;
+#endif
   if (my_system_page_size > 4096)
     return MAP_FAILED;
 # ifndef HAVE_PMEM
@@ -254,6 +258,10 @@ static void *log_mmap(os_file_t file,
   if (!read_only)
 #  ifdef HAVE_PMEM
     prot= PROT_READ | PROT_WRITE;
+
+#   ifdef __linux__ /* On Linux, we pretend that /dev/shm is PMEM */
+remap:
+#   endif
 #  else
     return MAP_FAILED;
 #  endif
@@ -269,8 +277,9 @@ static void *log_mmap(os_file_t file,
 
 #  ifdef HAVE_PMEM
 #   ifdef __linux__ /* On Linux, we pretend that /dev/shm is PMEM */
-  if (srv_operation < SRV_OPERATION_BACKUP)
+  if (flags != MAP_SHARED && srv_operation < SRV_OPERATION_BACKUP)
   {
+    flags= MAP_SHARED;
     struct stat st;
     if (!fstat(file, &st))
     {
@@ -282,6 +291,7 @@ static void *log_mmap(os_file_t file,
         is_pmem= st.st_dev == st_dev;
         if (!is_pmem)
           return ptr; /* MAP_FAILED */
+        goto remap;
       }
     }
   }
@@ -292,20 +302,17 @@ static void *log_mmap(os_file_t file,
 # endif
   return ptr;
 }
-#endif
 
 #if defined __linux__ || defined _WIN32
 /** Display a message about opening the log */
 ATTRIBUTE_COLD static void log_file_message()
 {
   sql_print_information("InnoDB: %s (block size=%u bytes)",
-# ifdef HAVE_INNODB_MMAP
                         log_sys.log_mmap
                         ? (log_sys.log_buffered
                            ? "Memory-mapped log"
                            : "Memory-mapped unbuffered log")
                         :
-# endif
                         log_sys.log_buffered
                         ? "Buffered log writes"
                         : "File system buffers for log disabled",
@@ -323,7 +330,7 @@ bool log_t::attach(log_file_t file, os_offset_t size)
 
   ut_ad(!buf);
   ut_ad(!flush_buf);
-#ifdef HAVE_INNODB_MMAP
+  ut_ad(!writer);
   if (size)
   {
 # ifdef HAVE_PMEM
@@ -345,7 +352,7 @@ bool log_t::attach(log_file_t file, os_offset_t size)
 # endif
       buf= static_cast<byte*>(ptr);
       max_buf_free= 1;
-      mtr_t::finisher_update();
+      writer_update();
 # ifdef HAVE_PMEM
       if (is_pmem)
         return true;
@@ -354,7 +361,6 @@ bool log_t::attach(log_file_t file, os_offset_t size)
     }
   }
   log_mmap= false;
-#endif
   buf= static_cast<byte*>(ut_malloc_dontdump(buf_size, PSI_INSTRUMENT_ME));
   if (!buf)
   {
@@ -388,12 +394,10 @@ bool log_t::attach(log_file_t file, os_offset_t size)
   TRASH_ALLOC(buf, buf_size);
   TRASH_ALLOC(flush_buf, buf_size);
   max_buf_free= buf_size / LOG_BUF_FLUSH_RATIO - LOG_BUF_FLUSH_MARGIN;
-  mtr_t::finisher_update();
+  writer_update();
   memset_aligned<512>(checkpoint_buf, 0, write_size);
 
-#ifdef HAVE_INNODB_MMAP
  func_exit:
-#endif
   log_file_message();
   return true;
 }
@@ -468,25 +472,19 @@ ATTRIBUTE_COLD static void log_close_failed(dberr_t err)
   ib::fatal() << "closing ib_logfile0 failed: " << err;
 }
 
-#ifdef HAVE_INNODB_MMAP
 void log_t::close_file(bool really_close)
-#else
-void log_t::close_file()
-#endif
 {
-#ifdef HAVE_INNODB_MMAP
   if (is_mmap())
   {
     ut_ad(!checkpoint_buf);
     ut_ad(!flush_buf);
     if (buf)
     {
-      my_munmap(buf, file_size);
+      my_munmap(buf, size_t(file_size));
       buf= nullptr;
     }
   }
   else
-#endif
   {
     ut_ad(!buf == !flush_buf);
     ut_ad(!buf == !checkpoint_buf);
@@ -501,9 +499,9 @@ void log_t::close_file()
     checkpoint_buf= nullptr;
   }
 
-#ifdef HAVE_INNODB_MMAP
+  writer= nullptr;
+
   if (really_close)
-#endif
     if (is_opened())
       if (const dberr_t err= log.close())
         log_close_failed(err);
@@ -635,14 +633,10 @@ log_t::resize_start_status log_t::resize_start(os_offset_t size) noexcept
       void *ptr= nullptr, *ptr2= nullptr;
       success= os_file_set_size(path.c_str(), resize_log.m_file, size);
       if (!success);
-#ifdef HAVE_INNODB_MMAP
+#ifdef HAVE_PMEM
       else if (is_mmap())
       {
-        ptr= ::log_mmap(resize_log.m_file,
-#ifdef HAVE_PMEM
-                        is_pmem,
-#endif
-                        size);
+        ptr= ::log_mmap(resize_log.m_file, is_pmem, size);
 
         if (ptr == MAP_FAILED)
           goto alloc_fail;
@@ -650,6 +644,7 @@ log_t::resize_start_status log_t::resize_start(os_offset_t size) noexcept
 #endif
       else
       {
+        ut_ad(!is_mmap());
         ptr= ut_malloc_dontdump(buf_size, PSI_INSTRUMENT_ME);
         if (ptr)
         {
@@ -689,10 +684,12 @@ log_t::resize_start_status log_t::resize_start(os_offset_t size) noexcept
              (lsn_t{write_size - 1} + start_lsn - first_lsn));
         else if (!is_opened())
           resize_log.close();
+
+        writer_update();
       }
-      resize_lsn.store(start_lsn, std::memory_order_relaxed);
       status= success ? RESIZE_STARTED : RESIZE_FAILED;
     }
+    resize_lsn.store(start_lsn, std::memory_order_relaxed);
   }
 
   log_resize_release();
@@ -717,28 +714,36 @@ void log_t::resize_abort() noexcept
 
   if (resize_in_progress() > 1)
   {
-    if (!is_mmap())
+#ifdef HAVE_PMEM
+    const bool is_mmap{this->is_mmap()};
+#else
+    constexpr bool is_mmap{false};
+#endif
+    if (!is_mmap)
     {
       ut_free_dodump(resize_buf, buf_size);
       ut_free_dodump(resize_flush_buf, buf_size);
       resize_flush_buf= nullptr;
     }
-#ifdef HAVE_INNODB_MMAP
     else
     {
       ut_ad(!resize_log.is_opened());
       ut_ad(!resize_flush_buf);
+#ifdef HAVE_PMEM
       if (resize_buf)
         my_munmap(resize_buf, resize_target);
+#endif /* HAVE_PMEM */
     }
-#endif
     if (resize_log.is_opened())
       resize_log.close();
     resize_buf= nullptr;
     resize_target= 0;
     resize_lsn.store(0, std::memory_order_relaxed);
+    std::string path{get_log_file_path("ib_logfile101")};
+    IF_WIN(DeleteFile(path.c_str()), unlink(path.c_str()));
   }
 
+  writer_update();
   log_resize_release();
 }
 
@@ -896,13 +901,13 @@ static size_t log_pad(lsn_t lsn, size_t pad, byte *begin, byte *extra)
 #endif
 
 #ifdef HAVE_PMEM
-void log_t::persist(lsn_t lsn, bool holding_latch) noexcept
+void log_t::persist(lsn_t lsn) noexcept
 {
   ut_ad(!is_opened());
   ut_ad(!write_lock.is_owner());
   ut_ad(!flush_lock.is_owner());
 #ifdef LOG_LATCH_DEBUG
-  ut_ad(holding_latch == latch_have_wr());
+  ut_ad(latch_have_any());
 #endif
 
   lsn_t old= flushed_to_disk_lsn.load(std::memory_order_relaxed);
@@ -910,9 +915,6 @@ void log_t::persist(lsn_t lsn, bool holding_latch) noexcept
   if (old >= lsn)
     return;
 
-  const bool latching{!holding_latch && resize_in_progress()};
-  if (UNIV_UNLIKELY(latching))
-    latch.rd_lock(SRW_LOCK_CALL);
   const size_t start(calc_lsn_offset(old));
   const size_t end(calc_lsn_offset(lsn));
 
@@ -936,13 +938,17 @@ void log_t::persist(lsn_t lsn, bool holding_latch) noexcept
     log_flush_notify(lsn);
     DBUG_EXECUTE_IF("crash_after_log_write_upto", DBUG_SUICIDE(););
   }
+}
 
-  if (UNIV_UNLIKELY(latching))
-    latch.rd_unlock();
+ATTRIBUTE_NOINLINE
+static void log_write_persist(lsn_t lsn) noexcept
+{
+  log_sys.latch.rd_lock(SRW_LOCK_CALL);
+  log_sys.persist(lsn);
+  log_sys.latch.rd_unlock();
 }
 #endif
 
-ATTRIBUTE_COLD ATTRIBUTE_NOINLINE
 void log_t::resize_write_buf(const byte *b, size_t length) noexcept
 {
   const size_t block_size_1= write_size - 1;
@@ -977,20 +983,24 @@ void log_t::resize_write_buf(const byte *b, size_t length) noexcept
                           b, offset, length) == DB_SUCCESS);
 }
 
-/** Write buf to ib_logfile0.
-@tparam release_latch whether to invoke latch.wr_unlock()
+/** Write buf to ib_logfile0 and possibly ib_logfile101.
+@tparam resizing whether to release latch and whether resize_in_progress()>1
 @return the current log sequence number */
-template<bool release_latch> inline lsn_t log_t::write_buf() noexcept
+template<log_t::resizing_and_latch resizing>
+inline __attribute__((always_inline))
+lsn_t log_t::write_buf() noexcept
 {
   ut_ad(latch_have_wr());
   ut_ad(!is_mmap());
   ut_ad(!srv_read_only_mode);
+  ut_ad(resizing == RETAIN_LATCH ||
+        (resizing == RESIZING) == (resize_in_progress() > 1));
 
   const lsn_t lsn{get_lsn(std::memory_order_relaxed)};
 
   if (write_lsn >= lsn)
   {
-    if (release_latch)
+    if (resizing != RETAIN_LATCH)
       latch.wr_unlock();
     ut_ad(write_lsn == lsn);
   }
@@ -1008,7 +1018,16 @@ template<bool release_latch> inline lsn_t log_t::write_buf() noexcept
     ut_ad(write_size_1 >= 511);
 
     const byte *const write_buf{buf};
-    const byte *const re_write_buf{resize_buf};
+    byte *const re_write_buf{resizing == NOT_RESIZING ? nullptr : resize_buf};
+    ut_ad(resizing == RETAIN_LATCH ||
+          (resizing == NOT_RESIZING) == !re_write_buf);
+    ut_ad(!re_write_buf == !resize_flush_buf);
+    if (resizing == RESIZING)
+#ifdef _MSC_VER
+      __assume(re_write_buf != nullptr);
+#else
+      if (!re_write_buf) __builtin_unreachable();
+#endif
     offset&= ~lsn_t{write_size_1};
 
     if (length <= write_size_1)
@@ -1020,13 +1039,14 @@ template<bool release_latch> inline lsn_t log_t::write_buf() noexcept
                         buf + length, flush_buf);
       ... /* TODO: Update the LSN and adjust other code. */
 #else
-# ifdef HAVE_valgrind
       MEM_MAKE_DEFINED(buf + length, (write_size_1 + 1) - length);
-      if (UNIV_LIKELY_NULL(re_write_buf))
-        MEM_MAKE_DEFINED(re_write_buf + length, (write_size_1 + 1) - length);
-# endif
-      buf[length]= 0; /* allow recovery to catch EOF faster */
+      buf[length]= 0; /* ensure that recovery catches EOF */
 #endif
+      if (UNIV_LIKELY_NULL(re_write_buf))
+      {
+        MEM_MAKE_DEFINED(re_write_buf + length, (write_size_1 + 1) - length);
+        re_write_buf[length]= 0;
+      }
       length= write_size_1 + 1;
     }
     else
@@ -1041,27 +1061,28 @@ template<bool release_latch> inline lsn_t log_t::write_buf() noexcept
         (We want to avoid memset() while holding exclusive log_sys.latch)
         This block will be overwritten later, once records beyond
         the current LSN are generated. */
-#ifdef HAVE_valgrind
-          MEM_MAKE_DEFINED(buf + length, (write_size_1 + 1) - new_buf_free);
-        if (UNIV_LIKELY_NULL(re_write_buf))
-          MEM_MAKE_DEFINED(re_write_buf + length, (write_size_1 + 1) -
-                           new_buf_free);
-#endif
+        MEM_MAKE_DEFINED(buf + length, (write_size_1 + 1) - new_buf_free);
         buf[length]= 0; /* allow recovery to catch EOF faster */
         length&= ~write_size_1;
         memcpy_aligned<16>(flush_buf, buf + length, (new_buf_free + 15) & ~15);
         if (UNIV_LIKELY_NULL(re_write_buf))
+        {
+          MEM_MAKE_DEFINED(re_write_buf + length, (write_size_1 + 1) -
+                           new_buf_free);
           memcpy_aligned<16>(resize_flush_buf, re_write_buf + length,
                              (new_buf_free + 15) & ~15);
+          re_write_buf[length + new_buf_free]= 0;
+        }
         length+= write_size_1 + 1;
       }
 
       std::swap(buf, flush_buf);
-      std::swap(resize_buf, resize_flush_buf);
+      if (UNIV_LIKELY_NULL(re_write_buf))
+        std::swap(resize_buf, resize_flush_buf);
     }
 
     write_to_log++;
-    if (release_latch)
+    if (resizing != RETAIN_LATCH)
       latch.wr_unlock();
 
     DBUG_PRINT("ib_log", ("write " LSN_PF " to " LSN_PF " at " LSN_PF,
@@ -1121,7 +1142,7 @@ wait and check if an already running write is covering the request.
 @param durable  whether the write needs to be durable
 @param callback log write completion callback */
 void log_write_up_to(lsn_t lsn, bool durable,
-                     const completion_callback *callback)
+                     const completion_callback *callback) noexcept
 {
   ut_ad(!srv_read_only_mode || log_sys.buf_free_ok());
   ut_ad(lsn != LSN_MAX);
@@ -1132,7 +1153,7 @@ void log_write_up_to(lsn_t lsn, bool durable,
   if (log_sys.is_mmap())
   {
     if (durable)
-      log_sys.persist(lsn, false);
+      log_write_persist(lsn);
     else
       ut_ad(!callback);
     return;
@@ -1159,7 +1180,7 @@ repeat:
   {
     ut_ad(!recv_no_log_write || srv_operation != SRV_OPERATION_NORMAL);
     log_sys.latch.wr_lock(SRW_LOCK_CALL);
-    pending_write_lsn= write_lock.release(log_sys.write_buf<true>());
+    pending_write_lsn= write_lock.release(log_sys.writer());
   }
 
   if (durable)
@@ -1176,6 +1197,23 @@ repeat:
   }
 }
 
+static lsn_t log_writer() noexcept
+{
+  return log_sys.write_buf<log_t::NOT_RESIZING>();
+}
+
+ATTRIBUTE_COLD static lsn_t log_writer_resizing() noexcept
+{
+  return log_sys.write_buf<log_t::RESIZING>();
+}
+
+void log_t::writer_update() noexcept
+{
+  ut_ad(latch_have_wr());
+  writer= resize_in_progress() ? log_writer_resizing : log_writer;
+  mtr_t::finisher_update();
+}
+
 /** Write to the log file up to the last log entry.
 @param durable  whether to wait for a durable write to complete */
 void log_buffer_flush_to_disk(bool durable)
@@ -1197,7 +1235,6 @@ ATTRIBUTE_COLD void log_write_and_flush_prepare()
          group_commit_lock::ACQUIRED);
 }
 
-#ifdef HAVE_INNODB_MMAP
 void log_t::clear_mmap()
 {
   if (!is_mmap() ||
@@ -1231,7 +1268,6 @@ void log_t::clear_mmap()
   }
   log_resize_release();
 }
-#endif
 
 /** Durably write the log up to log_sys.get_lsn(). */
 ATTRIBUTE_COLD void log_write_and_flush()
@@ -1239,11 +1275,11 @@ ATTRIBUTE_COLD void log_write_and_flush()
   ut_ad(!srv_read_only_mode);
 #ifdef HAVE_PMEM
   if (log_sys.is_mmap())
-    log_sys.persist(log_sys.get_lsn(), true);
+    log_sys.persist(log_sys.get_lsn());
   else
 #endif
   {
-    const lsn_t lsn{log_sys.write_buf<false>()};
+    const lsn_t lsn{log_sys.write_buf<log_t::RETAIN_LATCH>()};
     write_lock.release(lsn);
     log_flush(lsn);
   }
